@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Description:
 # Author: Pawel Krupa (paulfantom)
-# Author: Ilya Mashchenko (l2isbad)
+# Author: Ilya Mashchenko (ilyam8)
 # SPDX-License-Identifier: GPL-3.0-or-later
 
 import socket
@@ -16,6 +16,11 @@ else:
 from bases.FrameworkServices.SimpleService import SimpleService
 
 
+DEFAULT_CONNECT_TIMEOUT = 2.0
+DEFAULT_READ_TIMEOUT = 2.0
+DEFAULT_WRITE_TIMEOUT = 2.0
+
+
 class SocketService(SimpleService):
     def __init__(self, configuration=None, name=None):
         self._sock = None
@@ -31,6 +36,9 @@ class SocketService(SimpleService):
         self.__socket_config = None
         self.__empty_request = "".encode()
         SimpleService.__init__(self, configuration=configuration, name=name)
+        self.connect_timeout = configuration.get('connect_timeout', DEFAULT_CONNECT_TIMEOUT)
+        self.read_timeout = configuration.get('read_timeout', DEFAULT_READ_TIMEOUT)
+        self.write_timeout = configuration.get('write_timeout', DEFAULT_WRITE_TIMEOUT)
 
     def _socket_error(self, message=None):
         if self.unix_socket is not None:
@@ -75,15 +83,19 @@ class SocketService(SimpleService):
                                              keyfile=self.key,
                                              certfile=self.cert,
                                              server_side=False,
-                                             cert_reqs=ssl.CERT_NONE)
+                                             cert_reqs=ssl.CERT_NONE,
+                                             ssl_version=ssl.PROTOCOL_TLS,
+                                             )
             except (socket.error, ssl.SSLError) as error:
-                self.error('Failed to wrap socket.')
+                self.error('failed to wrap socket : {0}'.format(error))
                 self._disconnect()
                 self.__socket_config = None
                 return False
 
         try:
             self.debug('connecting socket to "{address}", port {port}'.format(address=sa[0], port=sa[1]))
+            self._sock.settimeout(self.connect_timeout)
+            self.debug('set socket connect timeout to: {0}'.format(self._sock.gettimeout()))
             self._sock.connect(sa)
         except (socket.error, ssl.SSLError) as error:
             self.error('Failed to connect to "{address}", port {port}, error: {error}'.format(address=sa[0],
@@ -109,6 +121,8 @@ class SocketService(SimpleService):
         try:
             self.debug('attempting DGRAM unix socket "{0}"'.format(self.unix_socket))
             self._sock = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
+            self._sock.settimeout(self.connect_timeout)
+            self.debug('set socket connect timeout to: {0}'.format(self._sock.gettimeout()))
             self._sock.connect(self.unix_socket)
             self.debug('connected DGRAM unix socket "{0}"'.format(self.unix_socket))
             return True
@@ -119,6 +133,8 @@ class SocketService(SimpleService):
         try:
             self.debug('attempting STREAM unix socket "{0}"'.format(self.unix_socket))
             self._sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+            self._sock.settimeout(self.connect_timeout)
+            self.debug('set socket connect timeout to: {0}'.format(self._sock.gettimeout()))
             self._sock.connect(self.unix_socket)
             self.debug('connected STREAM unix socket "{0}"'.format(self.unix_socket))
             return True
@@ -154,11 +170,6 @@ class SocketService(SimpleService):
             self._sock = None
             self.__socket_config = None
 
-        if self._sock is not None:
-            self._sock.setblocking(0)
-            self._sock.settimeout(5)
-            self.debug('set socket timeout to: {0}'.format(self._sock.gettimeout()))
-
     def _disconnect(self):
         """
         Close socket connection
@@ -169,8 +180,8 @@ class SocketService(SimpleService):
                 self.debug('closing socket')
                 self._sock.shutdown(2)  # 0 - read, 1 - write, 2 - all
                 self._sock.close()
-            except Exception:
-                pass
+            except Exception as error:
+                self.error(error)
             self._sock = None
 
     def _send(self, request=None):
@@ -181,6 +192,8 @@ class SocketService(SimpleService):
         # Send request if it is needed
         if self.request != self.__empty_request:
             try:
+                self.debug('set socket write timeout to: {0}'.format(self._sock.gettimeout()))
+                self._sock.settimeout(self.write_timeout)
                 self.debug('sending request: {0}'.format(request or self.request))
                 self._sock.send(request or self.request)
             except Exception as error:
@@ -201,6 +214,8 @@ class SocketService(SimpleService):
         while True:
             self.debug('receiving response')
             try:
+                self.debug('set socket read timeout to: {0}'.format(self._sock.gettimeout()))
+                self._sock.settimeout(self.read_timeout)
                 buf = self._sock.recv(4096)
             except Exception as error:
                 self._socket_error('failed to receive response: {0}'.format(error))
